/* slld_hal_zynq7000.c - SLLD Hardware Abstraction Layer Code for Xilinx Zynq SPIMC controller */

/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#include <stdio.h>
#include <unistd.h>
//#include "slld_targetspecific.h"
#include "slld.h"
#include "slld_hal.h"


int fd = 0;
static int qpi_flag = 0;

#ifdef USE_4B_ADDR_BAR
#define ADDR_SIZE ADDR_32BIT
#else
#define ADDR_SIZE ADDR_24BIT
#endif

// RamTable is used for set SPIMC command config register
// Because of the command difference, there should be a RamTable for each device.
Instruction RamTable[] =
{
//  Cmd      Has_Cmd      CmdMode                              Res       Has_Addr             AddrMode                          Addr_Len           Has_ModeBit         Has_Data             DataMode                        DummyCycCnt      RdWr          Res           Has_Dlp
//                  b0              b1-4                                    b5-6     b7                       b8-11                                 b12-13              b14                      b15                      b16-19                            b20-27                 b28             b29-30      b31
{0x00,  {{CMD_NO_,     CMD_SDR_SINGLE,                0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x01,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x02,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x03,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,      DATA_YES,          DATA_SDR_SINGLE,                   0x00,              OP_RD,        0x00,        DLP_NO_}}},
{0x04,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,                 MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                      0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x05,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{0x06,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x07,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x8
{-1, 0}, // 0x9
{-1, 0}, // 0xa
{0x0B,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0x0C,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xD
{-1, 0}, // 0xE
{-1, 0}, // 0xF
{-1, 0}, // 0x10
{-1, 0}, // 0x11
{0x12,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x13,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x14
{0x15,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x16
{-1, 0}, // 0x17
{-1, 0}, // 0x18
{-1, 0}, // 0x19
{-1, 0}, // 0x1a
{-1, 0}, // 0x1b
{-1, 0}, // 0x1c
{-1, 0}, // 0x1d
{-1, 0}, // 0x1e
{-1, 0}, // 0x1f
{0x20,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x21,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x22
{-1, 0}, // 0x23
{-1, 0}, // 0x24
{-1, 0}, // 0x25
{-1, 0}, // 0x26
{-1, 0}, // 0x27
{-1, 0}, // 0x28
{-1, 0}, // 0x29
{-1, 0}, // 0x2a
{0x2B,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x01,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x2c
{-1, 0}, // 0x2d
{-1, 0}, // 0x2e
{0x2F,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x30,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x31
{0x32,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x33,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{0x34,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x35,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{0x36,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x37
{0x38,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x39,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x3A
{0x3B,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_DUAL,                          0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0x3C,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_DUAL,                      0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0x3D,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x3E
{-1, 0}, // 0x3F
{-1, 0}, // 0x40
{0x41,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x01,              OP_RD,       0x00,        DLP_NO_}}},
{0x42,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x43,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x44,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x45
{-1, 0}, // 0x46
{-1, 0}, // 0x47
{0x48,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x49
{0x4A,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x4B,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x20,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x4C
{-1, 0}, // 0x4D
{-1, 0}, // 0x4E
{-1, 0}, // 0x4F
{0x50,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x51
{0x52,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x53,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x54
{-1, 0}, // 0x55
{-1, 0}, // 0x56
{-1, 0}, // 0x57
{-1, 0}, // 0x58
{-1, 0}, // 0x59
{0x5A,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x5B
{-1, 0}, // 0x5C
{-1, 0}, // 0x5D
{-1, 0}, // 0x5E
{-1, 0}, // 0x5F
{0x60,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x61
{-1, 0}, // 0x62
{-1, 0}, // 0x63
{-1, 0}, // 0x64
{0x65,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0x66,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x67
{-1, 0}, // 0x68
{-1, 0}, // 0x69
{-1, 0}, // 0x6A
{0x6B,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0x6C,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x6D
{-1, 0}, // 0x6E
{-1, 0}, // 0x6F
{-1, 0}, // 0x70
{0x71,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x72
{-1, 0}, // 0x73
{-1, 0}, // 0x74
{0x75,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x76
{0x77,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x78
{-1, 0}, // 0x79
{0x7A,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x7B
{-1, 0}, // 0x7C
{-1, 0}, // 0x7D
{0x7E,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x7F
{-1, 0}, // 0x80
{-1, 0}, // 0x81
{-1, 0}, // 0x82
{-1, 0}, // 0x83
{-1, 0}, // 0x84
{-1, 0}, // 0x85
{-1, 0}, // 0x86
{-1, 0}, // 0x87
{-1, 0}, // 0x88
{-1, 0}, // 0x89
{-1, 0}, // 0x8A
{-1, 0}, // 0x8B
{-1, 0}, // 0x8C
{-1, 0}, // 0x8D
{-1, 0}, // 0x8E
{-1, 0}, // 0x8F
{-1, 0}, // 0x90
{-1, 0}, // 0x91
{-1, 0}, // 0x92
{-1, 0}, // 0x93
{-1, 0}, // 0x94
{-1, 0}, // 0x95
{-1, 0}, // 0x96
{-1, 0}, // 0x97
{0x98,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0x99,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0x9A
{-1, 0}, // 0x9B
{-1, 0}, // 0x9C
{-1, 0}, // 0x9D
{-1, 0}, // 0x9E
{0x9F,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xA0
{-1, 0}, // 0xA1
{-1, 0}, // 0xA2
{-1, 0}, // 0xA3
{-1, 0}, // 0xA4
{-1, 0}, // 0xA5
{0xA6,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xA7,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x01,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xA8
{-1, 0}, // 0xA9
{-1, 0}, // 0xAA
{0xAB,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xAC
{-1, 0}, // 0xAD
{-1, 0}, // 0xAE
{0xAF,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_QUAD,                      0x00,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xB0
{-1, 0}, // 0xB1
{-1, 0}, // 0xB2
{-1, 0}, // 0xB3
{-1, 0}, // 0xB4
{-1, 0}, // 0xB5
{-1, 0}, // 0xB6
{0xB7,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xB8
{0xB9,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xBA
{0xBB,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_DUAL,                    ADDR_SIZE,     MODEBIT_YES,      DATA_YES,           DATA_SDR_DUAL,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0xBC,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_DUAL,                    ADDR_32BIT,     MODEBIT_YES,      DATA_YES,          DATA_SDR_DUAL,                     0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xBD
{-1, 0}, // 0xBE
{-1, 0}, // 0xBF
{-1, 0}, // 0xC0
{-1, 0}, // 0xC1
{-1, 0}, // 0xC2
{-1, 0}, // 0xC3
{-1, 0}, // 0xC4
{-1, 0}, // 0xC5
{-1, 0}, // 0xC6
{0xC7,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xC8
{-1, 0}, // 0xC9
{-1, 0}, // 0xCA
{-1, 0}, // 0xCB
{-1, 0}, // 0xCC
{-1, 0}, // 0xCD
{-1, 0}, // 0xCE
{-1, 0}, // 0xCF
{-1, 0}, // 0xD0
{-1, 0}, // 0xD1
{-1, 0}, // 0xD2
{-1, 0}, // 0xD3
{-1, 0}, // 0xD4
{-1, 0}, // 0xD5
{-1, 0}, // 0xD6
{-1, 0}, // 0xD7
{0xD8,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xD9
{-1, 0}, // 0xDA
{0xDB,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xDC,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xDD
{-1, 0}, // 0xDE
{-1, 0}, // 0xDF
{0xE0,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0xE1,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xE2,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xE3,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_32BIT,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xE4
{-1, 0}, // 0xE5
{-1, 0}, // 0xE6
{0xE7,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x01,              OP_RD,       0x00,        DLP_NO_}}},
{0xE8,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xE9,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xEA,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_YES,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{0xEB,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_QUAD,                    ADDR_SIZE,     MODEBIT_YES,     DATA_YES,           DATA_SDR_QUAD,                     0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0xEC,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_QUAD,                    ADDR_32BIT,     MODEBIT_YES,     DATA_YES,           DATA_SDR_QUAD,                     0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0xED,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_DDR_QUAD,                    ADDR_SIZE,     MODEBIT_YES,     DATA_YES,          DATA_DDR_QUAD,                      0x08,              OP_RD,       0x00,        DLP_NO_}}},
{0xEE,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_DDR_QUAD,                    ADDR_32BIT,     MODEBIT_YES,     DATA_YES,          DATA_DDR_QUAD,                      0x08,              OP_RD,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xEF
{-1, 0}, // 0xF0
{-1, 0}, // 0xF1
{-1, 0}, // 0xF2
{-1, 0}, // 0xF3
{-1, 0}, // 0xF4
{0xF5,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xF6
{-1, 0}, // 0xF7
{-1, 0}, // 0xF8
{-1, 0}, // 0xF9
{-1, 0}, // 0xFA
{0xFB,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_YES,         ADD_SDR_SINGLE,                  ADDR_SIZE,     MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
{-1, 0}, // 0xFC
{-1, 0}, // 0xFD
{-1, 0}, // 0xFE
{0xFF,  {{CMD_YES,     CMD_SDR_SINGLE,                  0x00,    ADDR_NO_,         ADD_SDR_SINGLE,                  0x00,               MODEBIT_NO_,     DATA_NO_,          DATA_SDR_SINGLE,                    0x00,              OP_WR,       0x00,        DLP_NO_}}},
};


// ***************************************************************************
//  API  for calling driver functions
//
// ***************************************************************************
SLLD_STATUS SPIMC_reg_read(unsigned int reg, unsigned int *val)
{
    int res;
    OSARGS Arg;

    Arg.dwArg1 = reg;
    res = ioctl(fd, IOCNR_SLLD_DEV_REG_READ, &Arg);
    if (res < 0) {
        printf("HAL: read register 0x%x failed\n", reg);
        return SLLD_E_HAL_ERROR;
    }
    *val = Arg.dwResult;

    return SLLD_OK;
}

SLLD_STATUS SPIMC_reg_write(unsigned int reg, unsigned int val)
{
    int res;
    OSARGS Arg;

    Arg.dwArg1 = reg;
    Arg.dwArg2 = val;
    res = ioctl(fd, IOCNR_SLLD_DEV_REG_WRITE, &Arg);
    if (res < 0) {
        printf("HAL: write register 0x%x to 0x%xfailed\n", reg, val);
        return SLLD_E_HAL_ERROR;
    }

    return SLLD_OK;
}

SLLD_STATUS SPIMC_cmd_read(unsigned short nOp, unsigned int nAddress, unsigned short nModeBit, void *pBuffer, unsigned int nBytes)
{
    int res;
    OSARGS Arg;

    Arg.dwArg1 = (unsigned int)nOp;
    Arg.dwArg2 = nAddress;
    Arg.dwArg3 = (unsigned int)nModeBit;
    Arg.dwArg4 = (unsigned int)pBuffer;
    Arg.dwArg5 = (unsigned int)nBytes;
    Arg.dwResult = 0;
    ioctl(fd, IOCNR_SLLD_DEV_CMD_READ, &Arg);
    res = Arg.dwResult;
    if (res != 0) {
        printf("HAL: cmd (0x%04x) read 0x%08x failed, %d\n", nOp, nAddress, res);
        return SLLD_E_HAL_ERROR;
    }

    return SLLD_OK;
}

SLLD_STATUS SPIMC_cmd_write(unsigned short nOp, unsigned int nAddress, void *pBuffer, unsigned int nBytes)
{
    int res;
    OSARGS Arg;

    Arg.dwArg1 = (unsigned int)nOp;
    Arg.dwArg2 = nAddress;
    Arg.dwArg3 = (unsigned int)pBuffer;
    Arg.dwArg4 = (unsigned int)nBytes;
    Arg.dwResult = 0;
    ioctl(fd, IOCNR_SLLD_DEV_CMD_WRITE, &Arg);
    res = Arg.dwResult;
    if (res != 0) {
        printf("HAL: cmd (0x%04x) write 0x%08x failed, %d\n", nOp, nAddress, res);
        return SLLD_E_HAL_ERROR;
    }

    return SLLD_OK;
}


// ***************************************************************************
//  FLASH_INIT - HAL initialization function
//
//  input : none
//
//  return value : status of the initialization - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_INIT(int sel, BYTE * name )
{
    int res;
    int MHz = 0; // use driver default setting
    OSARGS Arg;

    if (fd != 0) return SLLD_E_HAL_ERROR;
    
    fd = open("/dev/slt", O_RDWR);
    if (fd < 0)
    {
        printf("open /dev/slt failed!\n");
        return SLLD_E_HAL_ERROR;
    }

    Arg.dwArg1 = (unsigned int) name;
    Arg.dwArg2 = sel;
    Arg.dwArg3 = (unsigned int) MHz;
    ioctl(fd, IOCNR_SLLD_DEV_INIT, &Arg);
    res = Arg.dwResult;
    if (res != 0) {
        printf("HAL: flash_init failed\n");
        close(fd);
        fd = 0;
        return SLLD_E_HAL_ERROR;
    }

    qpi_flag = 0;

    return(SLLD_OK);
}

// ***************************************************************************
//  FLASH_DEINIT - HAL de-initialization function
//
//  input : none
//
//  return value : status of the initialization - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_DEINIT
(
void
)
{
    int res = SLLD_OK;
    OSARGS Arg;

    if (fd != 0) {
        res = ioctl(fd, IOCNR_SLLD_DEINIT, &Arg);
        if (res < 0) {
            printf("HAL: flash deinit failed\n");
            res =  SLLD_E_HAL_ERROR;
        }
        close(fd);
        fd = 0;
    } else {
        printf("HAL: deinit already done\n");
    }


    return res;
}


// ***************************************************************************
//  FLASH_READ - HAL read function
//
//  input : device_num            device number to which operation will be done
//          command               write a single command byte to flash
//          sys_addr              system address to be used
//          data_buffer           Pointer to the data buffer where to store the read data
//          Number_Of_Read_Bytes  number of bytes to be read
//
//  return value : status of the operation - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_READ
(
BYTE    device_num,                     /* device number */
BYTE     command,                        /* write a single command byte to flash */
ADDRESS  sys_addr,                       /* system address to be used */
BYTE    *data_buffer,                    /* Pointer to the data buffer containing data to be written */
int      Number_Of_Read_Bytes            /* number of bytes to be read */
)
{
    SLLD_STATUS res = SLLD_OK;
    unsigned int cmd_val = RamTable[command].SubCmdVal;
    SubCommand cmd_reg;

    // handle QPI mode
    if (qpi_flag == 1) {
        memcpy(&cmd_reg, &cmd_val, sizeof(unsigned int));
        if ((command != SPI_DDRQIOR_CMD) && (command != SPI_4DDRQIOR_CMD)) {
            cmd_reg.CmdMode = CMD_SDR_QUAD;
            cmd_reg.AddrMode = ADD_SDR_QUAD;
            cmd_reg.DataMode = DATA_SDR_QUAD;
        } else {
            cmd_reg.CmdMode = CMD_DDR_QUAD;
            cmd_reg.AddrMode = ADD_DDR_QUAD;
            cmd_reg.DataMode = DATA_DDR_QUAD;
        }
        memcpy(&cmd_val, &cmd_reg, sizeof(unsigned int));
    }

    // handle 0xAB when read ID
    if (command == SPI_RDID_AB_CMD) {
        memcpy(&cmd_reg, &cmd_val, sizeof(unsigned int));
        cmd_reg.DummyCycCnt = 24;
        cmd_reg.Has_Data = DATA_YES;
        cmd_reg.RdWr = OP_RD;
        memcpy(&cmd_val, &cmd_reg, sizeof(unsigned int));
    }

    // set command info register
    res = SPIMC_reg_write(SPIMC_REG_CMD_CONFIG_INFO, cmd_val);
    if (res != SLLD_OK) {
        printf("HAL: flash read set command info failed\n");
        return res;
    }
    
    res = SPIMC_cmd_read(command, sys_addr, modebit_char, data_buffer, Number_Of_Read_Bytes);

    return (res);
}


// ***************************************************************************
//  FLASH_WRITE - HAL write function
//
//  input : device_num               device number to which operation will be done
//          command                  write a single command byte to flash
//          sys_addr                 system address to be used
//          data_buffer              optional pointer to the data buffer
//          Number_Of_Written_Bytes  number of bytes to be written
//
//  return value : status of the operation - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_WRITE
(
BYTE    device_num,                   /* device number */
BYTE     command,                        /* write a single command byte to flash */
ADDRESS  sys_addr,                       /* system address to be used */
BYTE    *data_buffer,                    /* Pointer to the data buffer containing data to be written */
int      Number_Of_Written_Bytes         /* number of bytes to be written */
)
{
    SLLD_STATUS res = SLLD_OK;
    unsigned int cmd_val = RamTable[command].SubCmdVal;
    SubCommand cmd_reg;

     // handle QPI mode
    if (qpi_flag == 1) {
        memcpy(&cmd_reg, &cmd_val, sizeof(unsigned int));
        if ((command != SPI_DDRQIOR_CMD) && (command != SPI_4DDRQIOR_CMD)) {
            cmd_reg.CmdMode = CMD_SDR_QUAD;
            cmd_reg.AddrMode = ADD_SDR_QUAD;
            cmd_reg.DataMode = DATA_SDR_QUAD;
        } else {
            cmd_reg.CmdMode = CMD_DDR_QUAD;
            cmd_reg.AddrMode = ADD_DDR_QUAD;
            cmd_reg.DataMode = DATA_DDR_QUAD;
        }
        memcpy(&cmd_val, &cmd_reg, sizeof(unsigned int));
    }

    // set command info register
    res = SPIMC_reg_write(SPIMC_REG_CMD_CONFIG_INFO, cmd_val);
    if (res != SLLD_OK) {
        printf("HAL: flash write set command info failed\n");
        return res;
    }
    
    res = SPIMC_cmd_write(command, sys_addr, data_buffer, Number_Of_Written_Bytes);

    if (command == SPI_QPIEN_CMD)
        qpi_flag = 1;
    else if (command == SPI_QPIEX_CMD)
        qpi_flag = 0;
    
    return(res);
}


// ***************************************************************************
//  DelayMicroseconds - Performs a delay.
//
//  input : number of microseconds to delay the processor
//
//  return value : none
// ***************************************************************************
#define DELAY_1us 150
void os_DelayMicroSec(int microseconds)
{
    usleep(microseconds);

    return;
}

/*****************************************************************************/
