/* trace.h - Source Code for Cypress's LLD Virtual Logic Analyzer trace support */
/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#ifndef TRACE_INC_H
#define TRACE_INC_H

#ifdef __OLLD
#include "olld.h"
#elif __D2LLD
#include "d2lld.h"
#elif __OPLLD
#include "basic_lld.h"
#include "ext_lld.h"
#elif __SLLD
#include "slld.h"
#else
#include "lld.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

// Turn Trace Time stamps on/off
// #define PRINT_TIMESTAMP

typedef LLD_UINT32  TIMESTAMP;

#ifdef TRACE_NAND_SIGNAL
typedef enum
{
    TR_OP_READ = 1,
    TR_OP_WRITE,
    OP_GET_SIGNAL,
    OP_SET_SIGNAL
}OPERATION;
#else    // original
typedef enum
{
    TR_OP_READ = 1,
    TR_OP_WRITE
}OPERATION;
#endif

#ifdef TRACE_NAND_SIGNAL
void StoreTraceCycle(BYTE device_num, OPERATION op, FLASHDATA * base, ADDRESS offset, ADDRESS addr, FLASHDATA data);
#endif

#ifdef __D2LLD
typedef struct
{
    unsigned int trace_sync;
    BYTE device_num;
    OPERATION op;
    FLASHDATA *   base;
    ADDRESS   offset;
    unsigned int src;
    unsigned int size;
#ifdef PRINT_TIMESTAMP
    TIMESTAMP timestamp;
#endif
}CYCLE;

#elif (defined __SLLD)
typedef struct
{
    unsigned int trace_sync;
    BYTE device_num;
    OPERATION    op;
    FLASHDATA    command;
    ADDRESS      sys_addr;
    FLASHDATA    data;
    int          Number_Of_Bytes;
#ifdef PRINT_TIMESTAMP
    TIMESTAMP    timestamp;
#endif
}CYCLE;

#else
typedef struct
{
    unsigned int trace_sync;
    BYTE device_num,
    OPERATION op;
    FLASHDATA *   base;
    ADDRESS   offset;
    ADDRESS   addr;
    FLASHDATA data;
#ifdef PRINT_TIMESTAMP
    TIMESTAMP timestamp;
#endif
}CYCLE;
#endif

#define MAX_CYCLES 512

#ifndef TRACE_C
extern CYCLE trace[MAX_CYCLES];
extern int cycle;
extern int trace_wrapped;
#endif

#ifdef __SLLD
SLLD_STATUS FlashRead(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Read_Bytes);
SLLD_STATUS FlashWrite(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Written_Bytes);
extern SLLD_STATUS FLASH_READ(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Read_Bytes);
extern SLLD_STATUS FLASH_WRITE(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Written_Bytes);
#elif (defined __D2LLD)
extern void FlashWrite(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA * src, BYTECOUNT size);
extern void FlashRead(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA * buf, BYTECOUNT size);
#else
extern void FlashWrite(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA data);
extern FLASHDATA FlashRead(BYTE device_num, FLASHDATA * addr, ADDRESS offset);
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* TRACE_INC_H */
